﻿//-----------------------------------------------------------------------------------------
// File:  FuncTable.cs
// Title: Function Table
//
// Copyright Mike Christle 2016
//-----------------------------------------------------------------------------------------
//                               TERMS OF USE: MIT License
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this
// software and associated documentation files (the "Software"), to deal in the Software
// without restriction, including without limitation the rights to use, copy, modify,
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to the following
// conditions:
// 
// The above copyright notice and this permission notice shall be included in all copies
// or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
// CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
// OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//-----------------------------------------------------------------------------------------
// Date       Version
// ---------- --------
// 2016-06-01    1.0.0 Initial release
//-----------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PCC
{
    public struct FuncEntry
    {
        public VarType Type;
        public System.Collections.Generic.List<ValType> ParmList;
        public FuncEntry(VarType type)
        {
            Type = type;
            ParmList = new List<ValType>();
        }
        public void AddParm(ValType vt)
        {
            ParmList.Add(vt);
        }
    }

    class FuncTable
    {
        private Dictionary<string, FuncEntry> FTable = new Dictionary<string, FuncEntry>();
        private FuncEntry lastentry = new FuncEntry();

        public FuncTable()
        {
        }

        public void Clear()
        {
            FTable.Clear();
        }

        public FuncEntry this[string key] 
        { 
            get 
            {
                if (FTable.ContainsKey(key) == false)
                    throw new Exception("Function not declared: " + key);
                return FTable[key]; 
            } 
        }

        public void AddFunc(string name, VarType type)
        {
            lastentry = new FuncEntry(type);
            if (Contains(name) == false)
            {
                FTable.Add(name, lastentry);
            }
        }

        public void AddParm(int index, VarType type)
        {
            lastentry.AddParm(new ValType(index, type));
        }

        public void AddParm(ValType vt)
        {
            lastentry.AddParm(vt);
        }

        public bool Contains(string name)
        {
            return FTable.ContainsKey(name);
        }

        public VarType FuncType(string name)
        {
            if (FTable.ContainsKey(name) == false)
                return VarType.NONE;

            return FTable[name].Type;
        }
    }
}
