﻿//-----------------------------------------------------------------------------------------
// File:  Token.cs
// Title: Token Class
//
// Copyright Mike Christle 2016
//-----------------------------------------------------------------------------------------
//                               TERMS OF USE: MIT License
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this
// software and associated documentation files (the "Software"), to deal in the Software
// without restriction, including without limitation the rights to use, copy, modify,
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to the following
// conditions:
// 
// The above copyright notice and this permission notice shall be included in all copies
// or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
// CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
// OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//-----------------------------------------------------------------------------------------
// Date       Version
// ---------- --------
// 2016-06-01    1.0.0 Initial release
//-----------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PCC
{
    public enum Tokens
    {
        NONE, NOP, LABEL, CONSTANT, REAL_CONSTANT, ASSIGN, BIF, BIOP, DOTI, DOTD, DOTS,
        COMMA, COLON, SEMICOLON, PAREN_RIGHT, PAREN_LEFT, BRACKET_RIGHT, BRACKET_LEFT, BRACE_RIGHT, BRACE_LEFT,
        IF, ELSE, DO, WHILE, FOR, SWITCH, CASE, DEFAULT, RETURN, BREAK, CONTINUE, VAR_TYPE, GBYTE, GWORD, GLONG,
        LOGICAL_AND, LOGICAL_OR, LOGICAL_XOR, LOGICAL_NOT, BITWISE_NOT, BITWISE_AND, BITWISE_OR, BITWISE_XOR, BITWISE_AND_NOT,
        PLUS, MINUS, MULTIPLY, DIVIDE, MODULO, SHIFT_RIGHT, SHIFT_LEFT, ROTATE_RIGHT, ROTATE_LEFT,
        CMP_EQ, CMP_NE, CMP_GT, CMP_LT, CMP_GE, CMP_LE, LIMIT_MIN, LIMIT_MAX,
        WAITCNT, WAITPEQ, WAITPNE, WAITVID, LOCKNEW, LOCKSET, LOCKCLR, LOCKRET, PARITY
    }

    public enum VarType { NONE, VOID, INT, UINT, AINT, REAL, ANY, OPCODE };
    public enum BIFType
    {
        NONE, ABS, ABSN, ITOR, RTOI, CLKSET, CMPSUB, COGID, COGINIT, COGSTOP,
        REV, SIN, COS, TAN, ASIN, ACOS, ATAN, LOG, EXP, SQRT, ISQRT, CEIL, FLOOR, TRUNC, ROUND, FRACT
    };

    //---------------------------------------------------------------------------------------------------------------------------------------------
    public struct Token
    {
        public Tokens token;
        public Tokens assign_op;
        public BIFType bif_type;
        public VarType var_type;
        public int source_line;
        public int const_value;
        public string label;

        public Token(Tokens token, int source_line, Tokens assign_op = Tokens.NONE)
        {
            this.token = token;
            this.source_line = source_line;
            this.assign_op = assign_op;

            this.bif_type = BIFType.NONE;
            this.var_type = VarType.NONE;
            this.label = null;
            this.const_value = 0;
        }

        public Token(Tokens token, int source_line, BIFType bif_type)
        {
            this.token = token;
            this.source_line = source_line;
            this.bif_type = bif_type;

            this.var_type = VarType.NONE;
            this.label = null;
            this.assign_op = Tokens.NONE;
            this.const_value = 0;
        }

        public Token(Tokens token, int source_line, string label)
        {
            this.token = token;
            this.source_line = source_line;
            this.label = label;

            this.bif_type = BIFType.NONE;
            this.assign_op = Tokens.NONE;
            this.const_value = 0;
            this.var_type = VarType.NONE;
        }

        public Token(Tokens token, int source_line, string label, VarType var_type, int const_value)
        {
            this.token = token;
            this.source_line = source_line;
            this.label = label;
            this.var_type = var_type;
            this.const_value = const_value;

            this.bif_type = BIFType.NONE;
            this.assign_op = Tokens.NONE;
        }
    }
}
